/*
Copyright (C) 2002 Pedro Zorzenon Neto <pzn@vztech.com.br>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Do not forget to visit Free Software Foundation site: http://fsf.org

$Id: common.h,v 1.2 2003/06/30 17:20:24 pzn Exp $
*/

#ifndef _EEPCOMMON_H
#define _EEPCOMMON_H

/* to compile in DOS systems, define __dos__
 * if your compiler does not define it, use the line below */
/* #define __dos__ */

/* The DOS executable was compiled with Borland TurboC
 * version 2.01, option: compact memory model
 * and was tested on Win98 MS-DOS prompt. */

#endif
